//This source code is provided for personal and educational use only.
//Redistribution of source or derivitives is not permitted without the permission of the authour.
//Modifications may be made for educational purposes, provided that this license is not removed.
//Use this code at your OWN RISK! The authour accepts no responsibility for damage to hardware
//which may result from use of this code.
//
//Copyright Jan 2010 - Alex Jarkey
//http://www.buildtherobot.blogspot.com

#include "macro.h"

#include <util/delay.h>
#include <avr/interrupt.h>

#include "atmega128.h"

volatile char gTx0Queue[TX0QUEUESIZE];
volatile uint gTx0Back = 0;
volatile uint gTx0Front = 0;
volatile char gTx0Done;

volatile char gRx0Queue[RX0QUEUESIZE];
volatile uint gRx0Back = 0;
volatile uint gRx0Front = 0;

volatile extern uchar g_nextpose;
volatile extern int gA2D;

volatile unsigned long int gMsec = 0;

extern void sample(void);

//wCK UART0 Transmit ISR - Transmission Finished
ISR(USART0_TX_vect)
{
	TX0SendByte();
}

//wCK UART0 Receive ISR - Receive Data Packet
ISR(USART0_RX_vect)
{
	RX0GetByte();
}

//Polling interrupt. Occurs 1000 times per second.
ISR(TIMER0_OVF_vect) 
{
	gMsec++;
	gA2D=ADCH;
	sample();
}

//Timer for holding pose
ISR(TIMER1_OVF_vect) 
{
	g_nextpose = 1;
	TIMSK &= ~(1<<TOIE1);
	//TCCR1B=0x00;
}

ISR(ADC_vect)
{
	gA2D=ADCH;
}

//Place the first character in the transmit queue into send buffer
void TX0SendByte(void)
{
	if(!TX0Complete())
	{
		while(!(UCSR0A&(1<<UDRE)));		// Wait for data register to empty
		UDR0 = gTx0Queue[gTx0Front++];
	}
}

//Place a character at the end of the transmit queue
void TX0WriteByte(char byte)
{
	if(gTx0Back == gTx0Front)
	{
		gTx0Back = 0;
		gTx0Front = 0;
	}

	gTx0Queue[gTx0Back++] = byte;
}


//Take the character in the receive buffer and add it to the front of the receive queue
void RX0GetByte(void)
{
	gRx0Queue[gRx0Back++] = UDR0;
}

//Return the character at the front of the receive queue
char RX0ReadByte(void)
{
	char temp = gRx0Queue[gRx0Front++];
	if(gRx0Front >= gRx0Back)
	{
		gRx0Front = 0;
		gRx0Back = 0;
	}
	return temp;
}

//Initiate transmission
void TX0Start(void)
{
	UCSR0B |= (1<<TXCIE);		//Ensure Interrupts are Enabled
	UCSR0B |= (1<<TXEN);
	sei();
	TX0SendByte();
}

//True if transmission queue is empty
char TX0Complete(void)
{
	if(gTx0Front >= gTx0Back)
	{
		return 1;
	}
	return 0;
}

char RX0QueueLength(void)
{
	return gRx0Back - gRx0Front;
}

char * getPGMByteArray(char * source, char * destination, uint length)
{
	uint i = 0;

	if(source == NULL || destination == NULL)
	{
		//return NULL;
	}

	for(i = 0; i < length; i++)
	{
		destination[i] = pgm_read_byte(source+i);
	}
	return destination;
}

int * getPGMWordArray(int * source, int * destination, uint length)
{
	uint i = 0;

	if(source == NULL || destination == NULL)
	{
		//return NULL;
	}

	for(i = 0; i < length; i++)
	{
		destination[i] = pgm_read_word(source+i);
	}
	return destination;
}
