//This source code is provided for personal and educational use only.
//Redistribution of source or derivitives is not permitted without the permission of the authour.
//Modifications may be made for educational purposes, provided that this license is not removed.
//Use this code at your OWN RISK! The authour accepts no responsibility for damage to hardware
//which may result from use of this code.
//
//Copyright Jan 2010 - Alex Jarkey
//http://www.buildtherobot.blogspot.com

#include "macro.h"
#include <util/delay.h>
#include "robobuilder.h"

volatile uchar gPF1 = 0;
volatile uchar gPF2 = 0;
volatile uint gBtnMsec = 0;

volatile extern unsigned long int gMsec;

void queueWCKCmd3(char ID, char Data1, char Data2, char Data3)
{
	char CheckSum; 

	CheckSum = (ID^Data1^Data2^Data3)&0x7f;

	TX0WriteByte(HEADER);		

	TX0WriteByte(ID);		

	TX0WriteByte(Data1);		

	TX0WriteByte(Data2);	

	TX0WriteByte(Data3);	

	TX0WriteByte(CheckSum);		
	//Instructions placed in transmit buffer, but not sent
}


void queueWCKCmd2(char ID, char Data1, char Data2)
{
	char CheckSum; 

	CheckSum = (ID^Data1^Data2)&0x7f;

	TX0WriteByte(HEADER);		

	TX0WriteByte(ID);		

	TX0WriteByte(Data1);		

	TX0WriteByte(Data2);	

	TX0WriteByte(CheckSum);		
	//Instructions placed in transmit buffer, but not sent
}


void queueWCKCmd1(char ID, char Data1)
{
	char CheckSum; 

	CheckSum = (ID^Data1)&0x7f;

	TX0WriteByte(HEADER);		

	TX0WriteByte(ID);		

	TX0WriteByte(Data1);			

	TX0WriteByte(CheckSum);		
	//Instructions placed in transmit buffer, but not sent
}

//Watcher for PF1 and PF2. Counts duration of button press.
//Can be called at regular intervals or simply when required.
void updateButtons(void)
{
	static uint lMsec;
	uchar lbtmp = ~PINA&0x03, mask = (gPF1<<0)|(gPF2<1);
	uint dMsec = gMsec - lMsec; // Time since last check
	lMsec = gMsec;

	if(lbtmp&mask && !(~(lbtmp|~mask))) //Is button state unchanged?
	{
		gBtnMsec += dMsec;
	}
	else
	{
		gBtnMsec = 0;
	}

	gPF1 = lbtmp&0x01;
	gPF2 = (lbtmp&0x02)>>1;	
}

//Returns true if the state of PF1 and PF2 has been maintained for duration.
uchar readButtons(uchar pf1, uchar pf2, uint duration)
{
	uchar	btnCnt = 0;
	updateButtons();
	while(gPF1 == pf1 && gPF2 == pf2)	//Button state matches
	{
		updateButtons();
		if(++btnCnt>=10 && gBtnMsec >= duration){ return 1; }  	// loop 10 times to debounce	
	}
	return 0;
}

#define	CHARGE_ENABLE       PORTB |= (1<<0x04) 	// enable battery charge
#define	CHARGE_DISABLE      PORTB &= (1<<0x01)		// disable battery charge

void rechargeMode(void)
{
	int i;
	char continuous = 0x00;

	for(i = 0; i < 10; i++) // Flash LED to indicate mode has changed
	{
		PWR_LED1_ON;
		_delay_ms(100);
		debug(0x00);
		PWR_LED1_OFF;
		_delay_ms(100);
	}
	for(i = 0; i < 20 || continuous; i++)
	{
		if(readButtons(1,0,0)) { continuous ^= 0x01; } // PF1 will toggle continuous recharge mode
		CHARGE_ENABLE;
		_delay_ms(30);
		CHARGE_DISABLE;
		_delay_ms(500-30); // Pulse recharge voltage (trickle charge NiMH)
		debug(0x00);
		if(continuous) {
			PWR_LED1_OFF;
			i%2? PWR_LED2_ON : PWR_LED2_OFF; //RED = continuous mode
		}
		else {
			PWR_LED2_OFF;
			i%2? PWR_LED1_ON : PWR_LED1_OFF; //GREEN = 10 second burst mode
		}
	}
	CHARGE_DISABLE;
	for(i = 0; i < 10; i++) // Flash LED to indicate recharge has finished
	{
		PWR_LED2_ON;
		_delay_ms(100);
		debug(0x00);
		PWR_LED2_OFF;
		_delay_ms(100);
	}
}


